import O, { SVG as D, Browser as C, Canvas as F, Path as w, Handler as d, DomUtil as M, DomEvent as f, Util as x, point as y, LatLngBounds as T, Point as P, CircleMarker as k, LayerGroup as A, latLng as v, Polyline as I, GeoJSON as z, Rectangle as B } from "leaflet";
/**
* Drag/rotate/resize handler for [leaflet](http://leafletjs.com) vector features.
*
* @author Alexander Milevski <info@w8r.name>
* @license MIT
* @preserve
*/
D.include({
  /**
   * Reset transform matrix
   */
  _resetTransformPath: function(t) {
    t._path.setAttributeNS(null, "transform", "");
  },
  /**
   * Applies matrix transformation to SVG
   * @param {L.Path}         layer
   * @param {Array.<Number>} matrix
   */
  transformPath: function(t, i) {
    t._path.setAttributeNS(
      null,
      "transform",
      "matrix(" + i.join(" ") + ")"
    );
  }
});
D.include(
  C.vml ? {
    /**
     * Reset transform matrix
     */
    _resetTransformPath: function(t) {
      t._skew && (t._skew.on = !1, t._path.removeChild(t._skew), t._skew = null);
    },
    /**
     * Applies matrix transformation to VML
     * @param {L.Path}         layer
     * @param {Array.<Number>} matrix
     */
    transformPath: function(t, i) {
      let s = t._skew;
      s || (s = D.create("skew"), t._path.appendChild(s), s.style.behavior = "url(#default#VML)", t._skew = s);
      const a = i[0].toFixed(8) + " " + i[1].toFixed(8) + " " + i[2].toFixed(8) + " " + i[3].toFixed(8) + " 0 0", n = Math.floor(i[4]).toFixed() + ", " + Math.floor(i[5]).toFixed(), o = this._path.style;
      let r = parseFloat(o.left), e = parseFloat(o.top), h = parseFloat(o.width), l = parseFloat(o.height);
      isNaN(r) && (r = 0), isNaN(e) && (e = 0), (isNaN(h) || !h) && (h = 1), (isNaN(l) || !l) && (l = 1);
      const p = (-r / h - 0.5).toFixed(8) + " " + (-e / l - 0.5).toFixed(8);
      s.on = "f", s.matrix = a, s.origin = p, s.offset = n, s.on = !0;
    }
  } : {}
);
function X() {
  return !0;
}
F.include({
  /**
   * Do nothing
   * @param  {L.Path} layer
   */
  _resetTransformPath: function(t) {
    this._containerCopy && (delete this._containerCopy, t._containsPoint_ && (t._containsPoint = t._containsPoint_, delete t._containsPoint_, this._requestRedraw(t)));
  },
  /**
   * Algorithm outline:
   *
   * 1. pre-transform - clear the path out of the canvas, copy canvas state
   * 2. at every frame:
   *    2.1. save
   *    2.2. redraw the canvas from saved one
   *    2.3. transform
   *    2.4. draw path
   *    2.5. restore
   * 3. Repeat
   *
   * @param  {L.Path}         layer
   * @param  {Array.<Number>} matrix
   */
  transformPath: function(t, i) {
    let s = this._containerCopy;
    const a = this._ctx;
    let n;
    const o = C.retina ? 2 : 1, r = this._bounds, e = r.getSize(), h = r.min;
    s || (s = this._containerCopy = document.createElement("canvas"), n = s.getContext("2d"), s.width = o * e.x, s.height = o * e.y, this._removePath(t), this._redraw(), n.translate(o * r.min.x, o * r.min.y), n.drawImage(this._container, 0, 0), this._initPath(t), t._containsPoint_ = t._containsPoint, t._containsPoint = X), a.save(), a.clearRect(h.x, h.y, e.x * o, e.y * o), a.setTransform(1, 0, 0, 1, 0, 0), a.restore(), a.save(), a.drawImage(this._containerCopy, 0, 0, e.x, e.y), a.transform.apply(a, i), this._drawing = !0, t._updatePath(), this._drawing = !1, a.restore();
  }
});
/**
 * Leaflet vector features drag functionality
 * @author Alexander Milevski <info@w8r.name>
 * @preserve
 */
w.include({
  /**
   * Applies matrix transformation to SVG
   * @param {Array.<Number>?} matrix
   */
  _transform: function(t) {
    return this._renderer && (t ? this._renderer.transformPath(this, t) : (this._renderer._resetTransformPath(this), this._update())), this;
  },
  /**
   * Check if the feature was dragged, that'll supress the click event
   * on mouseup. That fixes popups for example
   *
   * @param  {MouseEvent} e
   */
  _onMouseClick: function(t) {
    this.dragging && this.dragging.moved() || this._map.dragging && this._map.dragging.moved() || this._fireMouseEvent(t);
  }
});
const Y = {
  mousedown: "mouseup",
  touchstart: "touchend",
  pointerdown: "touchend",
  MSPointerDown: "touchend"
}, Z = {
  mousedown: "mousemove",
  touchstart: "touchmove",
  pointerdown: "touchmove",
  MSPointerDown: "touchmove"
};
function W(t, i) {
  const s = t.x - i.x, a = t.y - i.y;
  return Math.sqrt(s * s + a * a);
}
d.PathDrag = d.extend(
  /** @lends  L.Path.Drag.prototype */
  {
    statics: {
      DRAGGING_CLS: "leaflet-path-draggable"
    },
    /**
     * @param  {L.Path} path
     * @constructor
     */
    initialize: function(t) {
      this._path = t, this._matrix = null, this._startPoint = null, this._dragStartPoint = null, this._mapDraggingWasEnabled = !1, this._path._dragMoved = !1;
    },
    /**
     * Enable dragging
     */
    addHooks: function() {
      this._path.on("mousedown", this._onDragStart, this), this._path.options.className = this._path.options.className ? this._path.options.className + " " + d.PathDrag.DRAGGING_CLS : d.PathDrag.DRAGGING_CLS, this._path._path && M.addClass(this._path._path, d.PathDrag.DRAGGING_CLS);
    },
    /**
     * Disable dragging
     */
    removeHooks: function() {
      this._path.off("mousedown", this._onDragStart, this), this._path.options.className = this._path.options.className.replace(
        new RegExp("\\s+" + d.PathDrag.DRAGGING_CLS),
        ""
      ), this._path._path && M.removeClass(this._path._path, d.PathDrag.DRAGGING_CLS);
    },
    /**
     * @return {Boolean}
     */
    moved: function() {
      return this._path._dragMoved;
    },
    /**
     * Start drag
     * @param  {L.MouseEvent} evt
     */
    _onDragStart: function(t) {
      const i = t.originalEvent._simulated ? "touchstart" : t.originalEvent.type;
      this._mapDraggingWasEnabled = !1, this._startPoint = t.containerPoint.clone(), this._dragStartPoint = t.containerPoint.clone(), this._matrix = [1, 0, 0, 1, 0, 0], f.stop(t.originalEvent), M.addClass(this._path._renderer._container, "leaflet-interactive"), f.on(document, Z[i], this._onDrag, this).on(
        document,
        Y[i],
        this._onDragEnd,
        this
      ), this._path._map.dragging.enabled() && (this._path._map.dragging.disable(), this._mapDraggingWasEnabled = !0), this._path._dragMoved = !1, this._path._popup && this._path._popup.close(), this._replaceCoordGetters(t);
    },
    /**
     * Dragging
     * @param  {L.MouseEvent} evt
     */
    _onDrag: function(t) {
      f.stop(t);
      const i = t.touches && t.touches.length >= 1 ? t.touches[0] : t, s = this._path._map.mouseEventToContainerPoint(i);
      if (t.type === "touchmove" && !this._path._dragMoved && this._dragStartPoint.distanceTo(s) <= this._path._map.options.tapTolerance)
        return;
      const a = s.x, n = s.y, o = a - this._startPoint.x, r = n - this._startPoint.y;
      (o || r) && (this._path._dragMoved || (this._path._dragMoved = !0, this._path.options.interactive = !1, this._path._map.dragging._draggable._moved = !0, this._path.fire("dragstart", t), this._path.bringToFront()), this._matrix[4] += o, this._matrix[5] += r, this._startPoint.x = a, this._startPoint.y = n, this._path.fire("predrag", t), this._path._transform(this._matrix), this._path.fire("drag", t));
    },
    /**
     * Dragging stopped, apply
     * @param  {L.MouseEvent} evt
     */
    _onDragEnd: function(t) {
      const i = this._path._map.mouseEventToContainerPoint(t), s = this.moved();
      if (s && (this._transformPoints(this._matrix), this._path._updatePath(), this._path._project(), this._path._transform(null), f.stop(t)), f.off(document, "mousemove touchmove", this._onDrag, this), f.off(document, "mouseup touchend", this._onDragEnd, this), this._restoreCoordGetters(), s) {
        this._path.fire("dragend", {
          distance: W(this._dragStartPoint, i)
        });
        const a = this._path._containsPoint;
        this._path._containsPoint = x.falseFn, x.requestAnimFrame(function() {
          this._path._dragMoved = !1, this._path.options.interactive = !0, this._path._containsPoint = a;
        }, this);
      }
      this._mapDraggingWasEnabled && this._path._map.dragging.enable();
    },
    /**
     * Applies transformation, does it in one sweep for performance,
     * so don't be surprised about the code repetition.
     *
     * [ x ]   [ a  b  tx ] [ x ]   [ a * x + b * y + tx ]
     * [ y ] = [ c  d  ty ] [ y ] = [ c * x + d * y + ty ]
     *
     * @param {Array.<Number>} matrix
     */
    _transformPoints: function(t, i) {
      const s = this._path, a = L.point(t[4], t[5]), n = s._map.options.crs, o = n.transformation, r = n.scale(s._map.getZoom()), e = n.projection, h = o.untransform(a, r).subtract(o.untransform(y(0, 0), r)), l = !i;
      if (s._bounds = new T(), s._point)
        i = e.unproject(
          e.project(s._latlng)._add(h)
        ), l && (s._latlng = i, s._point._add(a));
      else if (s._rings || s._parts) {
        const p = s._rings || s._parts;
        let c = s._latlngs;
        i = i || c, x.isArray(c[0]) || (c = [c], i = [i]);
        for (let _ = 0, m = p.length; _ < m; _++) {
          i[_] = i[_] || [];
          for (let g = 0, N = p[_].length; g < N; g++) {
            const R = c[_][g];
            i[_][g] = e.unproject(
              e.project(R)._add(h)
            ), l && (s._bounds.extend(c[_][g]), p[_][g]._add(a));
          }
        }
      }
      return i;
    },
    /**
     * If you want to read the latlngs during the drag - your right,
     * but they have to be transformed
     */
    _replaceCoordGetters: function() {
      this._path.getLatLng ? (this._path.getLatLng_ = this._path.getLatLng, this._path.getLatLng = x.bind(function() {
        return this.dragging._transformPoints(this.dragging._matrix, {});
      }, this._path)) : this._path.getLatLngs && (this._path.getLatLngs_ = this._path.getLatLngs, this._path.getLatLngs = x.bind(function() {
        return this.dragging._transformPoints(this.dragging._matrix, []);
      }, this._path));
    },
    /**
     * Put back the getters
     */
    _restoreCoordGetters: function() {
      this._path.getLatLng_ ? (this._path.getLatLng = this._path.getLatLng_, delete this._path.getLatLng_) : this._path.getLatLngs_ && (this._path.getLatLngs = this._path.getLatLngs_, delete this._path.getLatLngs_);
    }
  }
);
d.PathDrag.makeDraggable = function(t) {
  return t.dragging = new d.PathDrag(t), t;
};
w.prototype.makeDraggable = function() {
  return d.PathDrag.makeDraggable(this);
};
w.addInitHook(function() {
  this.options.draggable ? (this.options.interactive = !0, this.dragging ? this.dragging.enable() : (d.PathDrag.makeDraggable(this), this.dragging.enable())) : this.dragging && this.dragging.disable();
});
const U = d.PathDrag, H = (t, i, s) => {
  var a = 1 + s / t.distanceTo(i);
  return new L.Point(
    t.x + (i.x - t.x) * a,
    t.y + (i.y - t.y) * a
  );
}, E = (...t) => {
  var i = 1, s, a, n = t[i];
  function o(e) {
    return Object.prototype.toString.call(e) === "[object Object]";
  }
  for (var r = t[0]; n; ) {
    n = t[i++];
    for (s in n)
      n.hasOwnProperty(s) && (a = n[s], o(a) && o(r[s]) ? r[s] = L.PathTransform.merge(r[s], a) : r[s] = a);
  }
  return r;
}, V = /* @__PURE__ */ Object.freeze(/* @__PURE__ */ Object.defineProperty({
  __proto__: null,
  merge: E,
  pointOnLine: H
}, Symbol.toStringTag, { value: "Module" }));
class u {
  constructor(i, s, a, n, o, r) {
    this._matrix = [i, s, a, n, o, r];
  }
  /**
   * @param  {L.Point} point
   * @return {L.Point}
   */
  transform(i) {
    return this._transform(i.clone());
  }
  /**
   * Destructive
   *
   * [ x ] = [ a  b  tx ] [ x ] = [ a * x + b * y + tx ]
   * [ y ] = [ c  d  ty ] [ y ] = [ c * x + d * y + ty ]
   *
   * @param  {L.Point} point
   * @return {L.Point}
   */
  _transform(i) {
    const s = this._matrix, a = i.x, n = i.y;
    return i.x = s[0] * a + s[1] * n + s[4], i.y = s[2] * a + s[3] * n + s[5], i;
  }
  /**
   * @param  {L.Point} point
   * @return {L.Point}
   */
  untransform(i) {
    const s = this._matrix;
    return new P(
      (i.x / s[0] - s[4]) / s[0],
      (i.y / s[2] - s[5]) / s[2]
    );
  }
  /**
   * @return {L.Matrix}
   */
  clone() {
    const i = this._matrix;
    return new u(
      i[0],
      i[1],
      i[2],
      i[3],
      i[4],
      i[5]
    );
  }
  /**
   * @param {L.Point=|Number=} translate
   * @return {L.Matrix|L.Point}
   */
  translate(i) {
    if (i === void 0)
      return new P(this._matrix[4], this._matrix[5]);
    let s, a;
    return typeof i == "number" ? s = a = i : (s = i.x, a = i.y), this._add(1, 0, 0, 1, s, a);
  }
  /**
   * @param {L.Point=|Number=} scale
   * @return {L.Matrix|L.Point}
   */
  scale(i, s) {
    if (i === void 0)
      return new P(this._matrix[0], this._matrix[3]);
    let a, n;
    return s = s || new P(0, 0), typeof i == "number" ? a = n = i : (a = i.x, n = i.y), this._add(a, 0, 0, n, s.x, s.y)._add(
      1,
      0,
      0,
      1,
      -s.x,
      -s.y
    );
  }
  /**
   * m00  m01  x - m00 * x - m01 * y
   * m10  m11  y - m10 * x - m11 * y
   * @param {Number}   angle
   * @param {L.Point=} origin
   * @return {L.Matrix}
   */
  rotate(i, s) {
    const a = Math.cos(i), n = Math.sin(i);
    return s = s || new P(0, 0), this._add(a, n, -n, a, s.x, s.y)._add(
      1,
      0,
      0,
      1,
      -s.x,
      -s.y
    );
  }
  /**
   * Invert rotation
   * @return {L.Matrix}
   */
  flip() {
    return this._matrix[1] *= -1, this._matrix[2] *= -1, this;
  }
  /**
   * @param {Number|L.Matrix} a
   * @param {Number} b
   * @param {Number} c
   * @param {Number} d
   * @param {Number} e
   * @param {Number} f
   */
  _add(i, s, a, n, o, r) {
    const e = [[], [], []];
    let h = this._matrix;
    const l = [
      [h[0], h[2], h[4]],
      [h[1], h[3], h[5]],
      [0, 0, 1]
    ];
    let p = [
      [i, a, o],
      [s, n, r],
      [0, 0, 1]
    ], c;
    i && i instanceof u && (h = i._matrix, p = [
      [h[0], h[2], h[4]],
      [h[1], h[3], h[5]],
      [0, 0, 1]
    ]);
    for (let _ = 0; _ < 3; _++)
      for (let m = 0; m < 3; m++) {
        c = 0;
        for (let g = 0; g < 3; g++)
          c += l[_][g] * p[g][m];
        e[_][m] = c;
      }
    return this._matrix = [
      e[0][0],
      e[1][0],
      e[0][1],
      e[1][1],
      e[0][2],
      e[1][2]
    ], this;
  }
}
const b = (t, i, s, a, n, o) => new u(t, i, s, a, n, o), q = [
  "nesw-resize",
  "nwse-resize",
  "nesw-resize",
  "nwse-resize"
], S = k.extend({
  options: {
    className: "leaflet-path-transform-handler"
  },
  onAdd: function(t) {
    k.prototype.onAdd.call(this, t), this._path && this.options.setCursor && (this._path.style.cursor = q[this.options.index]);
  }
}), j = S.extend({
  options: {
    className: "leaflet-path-transform-handler transform-handler--rotate"
  },
  onAdd(t) {
    k.prototype.onAdd.call(this, t), this._path && this.options.setCursor && (this._path.style.cursor = "all-scroll");
  }
}), G = d.extend({
  options: {
    rotation: !0,
    scaling: !0,
    uniformScaling: !0,
    maxZoom: 22,
    // edge handlers
    handlerOptions: {
      radius: 5,
      fillColor: "#ffffff",
      color: "#202020",
      fillOpacity: 1,
      weight: 2,
      opacity: 0.7,
      setCursor: !0
    },
    // rectangle
    boundsOptions: {
      weight: 1,
      opacity: 1,
      dashArray: [3, 3],
      fill: !1,
      noClip: !0
    },
    // rotation handler
    rotateHandleOptions: {
      weight: 1,
      opacity: 1,
      setCursor: !0
    },
    // rotation handle length
    handleLength: 20,
    // maybe I'll add skewing in the future
    edgesCount: 4,
    handleClass: S,
    rotateHandleClass: j
  },
  /**
   * @class L.Handler.PathTransform
   * @constructor
   * @param  {L.Path} path
   */
  initialize(t) {
    this._path = t, this._map = null, this._activeMarker = null, this._originMarker = null, this._rotationMarker = null, this._rotationOrigin = null, this._scaleOrigin = null, this._angle = 0, this._scale = y(1, 1), this._initialDist = 0, this._initialDistX = 0, this._initialDistY = 0, this._rotationStart = null, this._rotationOriginPt = null, this._matrix = new u(1, 0, 0, 1, 0, 0), this._projectedMatrix = new u(1, 0, 0, 1, 0, 0), this._handlersGroup = null, this._rect = null, this._handlers = [], this._handleLine = null;
  },
  /**
   * If the polygon is not rendered, you can transform it yourself
   * in the coordinates, and do it properly.
   * @param {Object=} options
   */
  enable(t) {
    this._path._map && (this._map = this._path._map, t && this.setOptions(t), d.prototype.enable.call(this));
  },
  /**
   * Init interactions and handlers
   */
  addHooks() {
    this._createHandlers(), this._path.on("dragstart", this._onDragStart, this).on("dragend", this._onDragEnd, this);
  },
  /**
   * Remove handlers
   */
  removeHooks() {
    this._hideHandlers(), this._path.off("dragstart", this._onDragStart, this).off("dragend", this._onDragEnd, this), this._handlersGroup = null, this._rect = null, this._handlers = [];
  },
  /**
   * Change editing options
   * @param {Object} options
   */
  setOptions(t) {
    const i = this._enabled;
    return i && this.disable(), this.options = E({}, G.prototype.options, t), i && this.enable(), this;
  },
  /**
   * @param  {Number}   angle
   * @param  {L.LatLng} origin
   * @return {L.Handler.PathTransform}
   */
  rotate(t, i) {
    return this.transform(t, null, i);
  },
  /**
   * @param  {L.Point|Number} scale
   * @param  {L.LatLng}       origin
   * @return {L.Handler.PathTransform}
   */
  scale(t, i) {
    return typeof t == "number" && (t = y(t, t)), this.transform(0, t, null, i);
  },
  /**
   * @param  {Number}    angle
   * @param  {L.Point}   scale
   * @param  {L.LatLng=} rotationOrigin
   * @param  {L.LatLng=} scaleOrigin
   * @return {L.Handler.PathTransform}
   */
  transform(t, i, s, a) {
    const n = this._path.getCenter();
    return s = s || n, a = a || n, this._map = this._path._map, this._transformPoints(
      this._path,
      t,
      i,
      s,
      a
    ), this._transformPoints(
      this._rect,
      t,
      i,
      s,
      a
    ), this._transformPoints(
      this._handleLine,
      t,
      i,
      s,
      a
    ), this._updateHandlers(), this;
  },
  /**
   * Update the polygon and handlers preview, no reprojection
   */
  _update() {
    const t = this._matrix;
    for (const i of this._handlers)
      i !== this._originMarker && (i._point = t.transform(i._initialPoint), i._updatePath());
    this._applyTransform(t.clone().flip()), this._path.fire("transform", { layer: this._path });
  },
  /**
   * @param  {L.Matrix} matrix
   */
  _applyTransform(t) {
    this._path._transform(t._matrix), this._rect._transform(t._matrix), this.options.rotation && this._handleLine._transform(t._matrix);
  },
  /**
   * Apply final transformation
   */
  _apply() {
    const t = this._map, i = this._matrix.clone(), s = this._angle, a = this._scale.clone();
    this._transformGeometries();
    for (const n of this._handlers)
      n._latlng = t.layerPointToLatLng(n._point), delete n._initialPoint, n.redraw();
    this._matrix = new u(1, 0, 0, 1, 0, 0), this._scale = y(1, 1), this._angle = 0, this._updateHandlers(), t.dragging.enable(), this._path.fire("transformed", {
      matrix: i,
      scale: a,
      rotation: s,
      // angle: angle * (180 / Math.PI),
      layer: this._path
    });
  },
  /**
   * Use this method to completely reset handlers, if you have changed the
   * geometry of transformed layer
   */
  reset() {
    this._enabled && (this._rect && (this._handlersGroup.removeLayer(this._rect), this._rect = this._getBoundingPolygon().addTo(this._handlersGroup)), this._updateHandlers());
  },
  /**
   * Recalculate rotation handlers position
   */
  _updateHandlers() {
    const t = this._handlersGroup;
    this._rectShape = this._rect.toGeoJSON(), this._handleLine && this._handlersGroup.removeLayer(this._handleLine), this._rotationMarker && this._handlersGroup.removeLayer(this._rotationMarker), this._handleLine = this._rotationMarker = null;
    for (let i = this._handlers.length - 1; i >= 0; i--)
      t.removeLayer(this._handlers[i]);
    this._createHandlers();
  },
  /**
   * Transform geometries separately
   */
  _transformGeometries() {
    this._path._transform(null), this._rect._transform(null), this._transformPoints(this._path), this._transformPoints(this._rect), this.options.rotation && (this._handleLine._transform(null), this._transformPoints(this._handleLine, this._angle, null, this._origin));
  },
  /**
   * @param {Number} angle
   * @param {Number} scale
   * @param {L.LatLng=} rotationOrigin
   * @param {L.LatLng=} scaleOrigin
   */
  _getProjectedMatrix(t, i, s, a) {
    const n = this._map, o = n.getMaxZoom() || this.options.maxZoom;
    let r = b(1, 0, 0, 1, 0, 0), e;
    return t = t || this._angle || 0, i = i || this._scale || y(1, 1), i.x === 1 && i.y === 1 || (a = a || this._scaleOrigin, e = n.project(a, o), r = r._add(b(1, 0, 0, 1, e.x, e.y))._add(b(i.x, 0, 0, i.y, 0, 0))._add(b(1, 0, 0, 1, -e.x, -e.y))), t && (s = s || this._rotationOrigin, e = n.project(s, o), r = r.rotate(t, e).flip()), r;
  },
  /**
   * @param  {L.LatLng} latlng
   * @param  {L.Matrix} matrix
   * @param  {L.Map}    map
   * @param  {Number}   zoom
   * @return {L.LatLng}
   */
  _transformPoint(t, i, s, a) {
    return s.unproject(i.transform(s.project(t, a)), a);
  },
  /**
   * Applies transformation, does it in one sweep for performance,
   * so don't be surprised about the code repetition.
   *
   * @param {L.Path}    path
   * @param {Number=}   angle
   * @param {L.Point=}  scale
   * @param {L.LatLng=} rotationOrigin
   * @param {L.LatLng=} scaleOrigin
   */
  _transformPoints(t, i, s, a, n) {
    const o = t._map, r = o.getMaxZoom() || this.options.maxZoom, e = this._projectedMatrix = this._getProjectedMatrix(
      i,
      s,
      a,
      n
    );
    if (t._point) {
      t._latlng = this._transformPoint(
        t._latlng,
        e,
        o,
        r
      );
      const h = this._getBoundingPolygon(), l = Math.floor(
        Math.abs(h._bounds._southWest.lat - h._bounds._northEast.lat) / 2
      );
      t.setRadius(l);
    } else if (t._rings || t._parts) {
      const h = t._rings;
      let l = t._latlngs;
      t._bounds = new T(), x.isArray(l[0]) || (l = [l]);
      for (let p = 0, c = h.length; p < c; p++)
        for (let _ = 0, m = h[p].length; _ < m; _++)
          l[p][_] = this._transformPoint(
            l[p][_],
            e,
            o,
            r
          ), t._bounds.extend(l[p][_]);
    }
    t._reset();
  },
  /**
   * Creates markers and handles
   */
  _createHandlers() {
    const t = this._map;
    if (this._handlersGroup = this._handlersGroup || new A().addTo(t), this._rect = this._rect || this._getBoundingPolygon().addTo(this._handlersGroup), this.options.scaling) {
      this._handlers = [];
      for (let i = 0; i < this.options.edgesCount; i++)
        this._handlers.push(
          this._createHandler(this._rect._latlngs[0][i], i * 2, i).addTo(
            this._handlersGroup
          )
        );
    }
    this.options.rotation && this._createRotationHandlers(), this._handlersGroup.getLayers().forEach(function(i) {
      i.bringToFront();
    });
  },
  /**
   * Rotation marker and small connectin handle
   */
  _createRotationHandlers() {
    const t = this._map, i = this._rect._latlngs[0], s = v(
      (i[0].lat + i[3].lat) / 2,
      (i[0].lng + i[3].lng) / 2
    ), a = v(
      (i[1].lat + i[2].lat) / 2,
      (i[1].lng + i[2].lng) / 2
    ), n = t.layerPointToLatLng(
      H(
        t.latLngToLayerPoint(s),
        t.latLngToLayerPoint(a),
        this.options.handleLength
      )
    );
    this._handleLine = new I(
      [a, n],
      this.options.rotateHandleOptions
    ).addTo(this._handlersGroup);
    const o = this.options.rotateHandleClass;
    this._rotationMarker = new o(
      n,
      this.options.handlerOptions
    ).addTo(this._handlersGroup).on("mousedown", this._onRotateStart, this), this._rotationOrigin = v(
      (a.lat + s.lat) / 2,
      (a.lng + s.lng) / 2
    ), this._handlers.push(this._rotationMarker);
  },
  /**
   * @return {L.LatLng}
   */
  _getRotationOrigin() {
    const t = this._rect._latlngs[0], i = t[0], s = t[2];
    return v((i.lat + s.lat) / 2, (i.lng + s.lng) / 2);
  },
  /**
   * Secure the rotation origin
   * @param  {Event} evt
   */
  _onRotateStart(t) {
    this._map.dragging.disable(), this._originMarker = null, this._rotationOriginPt = this._map.latLngToLayerPoint(
      this._getRotationOrigin()
    ), this._rotationStart = t.layerPoint, this._initialMatrix = this._matrix.clone();
    const i = t.originalEvent._simulated;
    console.log({ isTouch: i }), this._angle = 0, this._path._map.on(i ? "touchmove" : "mousemove", this._onRotate, this).on(i ? "touchend" : "mouseup", this._onRotateEnd, this), this._cachePoints(), this._path.fire("transformstart", { layer: this._path }).fire("rotatestart", { layer: this._path, rotation: 0 });
  },
  /**
   * @param  {Event} evt
   */
  _onRotate(t) {
    f.stop(t);
    const i = t.layerPoint, s = this._rotationStart, a = this._rotationOriginPt;
    this._angle = Math.atan2(i.y - a.y, i.x - a.x) - Math.atan2(s.y - a.y, s.x - a.x), this._matrix = this._initialMatrix.clone().rotate(this._angle, a).flip(), this._update(), this._path.fire("rotate", { layer: this._path, rotation: this._angle });
  },
  /**
   * @param  {Event} evt
   */
  _onRotateEnd() {
    this._path._map.off("mousemove touchmove", this._onRotate, this).off("mouseup touchend", this._onRotateEnd, this);
    const t = this._angle;
    this._apply(), this._path.fire("rotateend", { layer: this._path, rotation: t });
  },
  /**
   * @param  {Event} evt
   */
  _onScaleStart(t) {
    const i = t.target, s = this._map, a = t.originalEvent._simulated;
    s.dragging.disable(), this._activeMarker = i, this._originMarker = this._handlers[(i.options.index + 2) % 4], this._scaleOrigin = this._originMarker.getLatLng(), this._initialMatrix = this._matrix.clone(), this._cachePoints(), this._map.on(a ? "touchmove" : "mousemove", this._onScale, this).on(a ? "touchend" : "mouseup", this._onScaleEnd, this), this._initialDist = this._originMarker._point.distanceTo(
      this._activeMarker._point
    ), this._initialDistX = this._originMarker._point.x - this._activeMarker._point.x, this._initialDistY = this._originMarker._point.y - this._activeMarker._point.y, this._path.fire("transformstart", { layer: this._path }).fire("scalestart", { layer: this._path, scale: L.point(1, 1) }), this._handleLine && this._map.removeLayer(this._handleLine), this._rotationMarker && this._map.removeLayer(this._rotationMarker);
  },
  /**
   * @param  {Event} evt
   */
  _onScale(t) {
    const i = this._originMarker._point;
    let s, a;
    this.options.uniformScaling ? (s = i.distanceTo(t.layerPoint) / this._initialDist, a = s) : (s = (i.x - t.layerPoint.x) / this._initialDistX, a = (i.y - t.layerPoint.y) / this._initialDistY), this._scale = y(s, a), this._matrix = this._initialMatrix.clone().scale(this._scale, i), this._update(), this._path.fire("scale", {
      layer: this._path,
      scale: this._scale.clone()
    });
  },
  /**
   * Scaling complete
   * @param  {Event} evt
   */
  _onScaleEnd() {
    this._map.off("mousemove touchmove", this._onScale, this).off("mouseup touchend", this._onScaleEnd, this), this._handleLine && this._map.addLayer(this._handleLine), this._rotationMarker && this._map.addLayer(this._rotationMarker), this._apply(), this._path.fire("scaleend", {
      layer: this._path,
      scale: this._scale.clone()
    });
  },
  /**
   * Cache current handlers positions
   */
  _cachePoints() {
    this._handlersGroup.eachLayer((t) => t.bringToFront());
    for (const t of this._handlers)
      t._initialPoint = t._point.clone();
  },
  /**
   * Bounding polygon
   * @return {L.Polygon}
   */
  _getBoundingPolygon() {
    return this._rectShape ? z.geometryToLayer(
      this._rectShape,
      this.options.boundsOptions
    ) : new B(this._path.getBounds(), this.options.boundsOptions);
  },
  /**
   * Create corner marker
   * @param  {L.LatLng} latlng
   * @param  {Number}   type one of L.Handler.PathTransform.HandlerTypes
   * @param  {Number}   index
   * @return {L.Handler.PathTransform.Handle}
   */
  _createHandler(t, i, s) {
    const a = this.options.handleClass, n = new a(
      t,
      x.extend({}, this.options.handlerOptions, {
        className: "leaflet-drag-transform-marker drag-marker--" + s + " drag-marker--" + i,
        index: s,
        type: i
      })
    );
    return n.on("mousedown", this._onScaleStart, this), n;
  },
  /**
   * Hide(not remove) the handlers layer
   */
  _hideHandlers() {
    this._map.removeLayer(this._handlersGroup);
  },
  /**
   * Hide handlers and rectangle
   */
  _onDragStart() {
    this._hideHandlers();
  },
  /**
   * Drag rectangle, re-create handlers
   */
  _onDragEnd(t) {
    const i = this._rect, s = (t.layer ? t.layer : this._path).dragging._matrix.slice();
    i.dragging || (i.dragging = new U(i)), i.dragging.enable(), this._map.addLayer(i), i.dragging._transformPoints(s), i._updatePath(), i._project(), i.dragging.disable(), this._map.addLayer(this._handlersGroup), this._updateHandlers(), this._path.fire("transformed", {
      scale: L.point(1, 1),
      rotation: 0,
      matrix: new u(...s),
      translate: y(s[4], s[5]),
      layer: this._path
    });
  }
});
d.RotateHandle = j;
d.Handle = S;
w.addInitHook(function() {
  this.options.transform && (this.transform = new G(this, this.options.transform));
});
O.Handler.PathTransform = G;
export {
  u as Matrix,
  G as Transform,
  V as Util,
  b as matrix
};
